package com.dsk.flink.dsc.common.function;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.streaming.api.functions.ProcessFunction;
import org.apache.flink.util.Collector;

import java.text.SimpleDateFormat;
import java.util.*;

public class MysqlDataTransferFunction extends ProcessFunction<JSONObject, Tuple3<String,String,Long>> {

    private static final String[] STR_SQL_TYPE = new String[]{"VARCHAR","CHAR","TINYBLOB","BLOB","MEDIUMBLOB","LONGBLOB","TINYTEXT","TEXT","MEDIUMTEXT","LONGTEXT","TIME","TIMESTAMP","JSON","json"};

    private static String tranferInsertSql(String table, JSONObject dataObj, JSONObject mysqlType) {

        Set<String> columnSet = mysqlType.keySet();
        StringBuilder sb = new StringBuilder();
        for (String s : columnSet) {
            sb.append("`").append(s).append("`,");
        }
        List<String> valueList = new ArrayList<>();
        for (String col : columnSet) {
            valueList.add(getValueString(dataObj,col,mysqlType.getString(col)));
        }
        sb.setLength(sb.length()-1);
        String columnString = sb.toString();
        String valueString = String.join(",",valueList);

        //return String.format("INSERT INTO %s (%s) values (%s) ON DUPLICATE KEY UPDATE %s;",table,columnString,valueString,updateString);
        return String.format("REPLACE INTO %s (%s) values (%s);",table,columnString,valueString);
    }


    private String transferDeleteSql(String table, JSONObject dataObj, JSONObject mysqlType, Set<String> pkNameSet) {

        List<String> whereList = new ArrayList<>();
        for (String pk : pkNameSet) {
            String whereString = pk.concat(" = ").concat(getValueString(dataObj,pk,mysqlType.getString(pk)));
            whereList.add(whereString);
        }
        String whereString = String.join(" and ",whereList);
        return String.format("DELETE FROM %s WHERE %s",table,whereString);
    }

    /**
     * @author shezaixing
     * @date 2023/12/7 14:23
     * @description 判断拼接字符串时类型（是否需要加上引号）
     *
     */
    private static String getValueString(JSONObject dataObj,String columnKey,String mysqlType){

        if(null == dataObj.get(columnKey)){
            return "null";
        }

        //需要处理成字符串加引号的类型
        if(Arrays.asList(STR_SQL_TYPE).contains(mysqlType.toUpperCase())){
            return String.format("'%s'", dataObj.getString(columnKey).replace("\\","\\\\").replace("'", "\\'") );
        }

        //时间字段处理
        if("DATE".equalsIgnoreCase(mysqlType) || "DATETIME".equalsIgnoreCase(mysqlType)){
            String date = "DATETIME".equalsIgnoreCase(mysqlType) ? DateUtil.format(dataObj.getDate(columnKey),"yyyy-MM-dd HH:mm:ss") : DateUtil.format(dataObj.getDate(columnKey),"yyyy-MM-dd");
            return String.format("\"%s\"",date);
        }

        return dataObj.getString(columnKey);
    }

    @Override
    public void processElement(JSONObject value, Context ctx, Collector<Tuple3<String, String, Long>> out) throws Exception {
        //返回数据集合
        String type = value.getString("type");
        JSONArray dataList = value.getJSONArray("data");
        JSONObject mysqlType = value.getJSONObject("mysqlType");
        String table = value.getString("table");
        JSONArray pkNames = value.getJSONArray("pkNames");
        Set<String> pkNameSet = new HashSet<>();
        long ts = value.getLong("ts") == null ? System.currentTimeMillis() : value.getLong("ts");
        if(CollUtil.isNotEmpty(pkNames)){
            pkNames.forEach(name -> pkNameSet.add(String.valueOf(name)));
        }
        String excueteSql;
        JSONObject dataObj = dataList.getJSONObject(0);

        /*Boolean logicalDelete = MapUtil.getBool(dbInfoMap, "logical_delete", false);
        if(logicalDelete){
            mysqlType.put("is_del", "int");
            dataObj.put("is_del", "DELETE".equals(type) ? 1 : 0);
        }*/

        //处理先后顺序
        //获取该条数据的表名和主键作为唯一的groupKey
        String groupKey = table;
        for (String pk : pkNameSet) {
            String pkValue = getValueString(dataObj, pk, mysqlType.getString(pk));
            groupKey = table.concat("-").concat(pkValue);
        }

        if("INSERT".equals(type) || "UPDATE".equals(type)){
            excueteSql = tranferInsertSql(table,dataObj,mysqlType);
        } else {
            excueteSql = transferDeleteSql(table,dataObj,mysqlType,pkNameSet);
        }
        out.collect(Tuple3.of(excueteSql,groupKey,ts));
        /*if (MapUtil.getBool(dbInfoMap, "log_enable", false)){
            String logSql = buildLogData(type, table, pkNameSet, dataObj, ts, value.toJSONString());
            ctx.output(toSlideTag,Tuple3.of(logSql,"dsc_cdc_log",ts));
        }*/
    }
}
